from osv import osv,fields
import time

class stock_inventory_line(osv.osv):
    _name="stock.inventory.line"
    _inherit="stock.inventory.line"
    _columns={
        "prodlot_id": fields.many2one("stock.production.lot","Production Lot",select=True),
        "tracking_id": fields.many2one("stock.tracking","Tracking Lot",select=True),
    }

    def on_change_product_id(self, cr, uid, ids, location_id, product, uom=False, prodlot=False, tracking=False):
        if not product:
            return {}
        if not uom:
            prod = self.pool.get('product.product').browse(cr, uid, [product])[0]
            uom = prod.uom_id.id
        amount=self.pool.get('stock.location')._product_get(cr, uid, location_id, [product], {'uom': uom, 'prodlot': prodlot, 'tracking': tracking})[product]
        result = {'product_qty':amount, 'product_uom':uom}
        return {'value':result}
stock_inventory_line()

class stock_inventory(osv.osv):
    _name="stock.inventory"
    _inherit="stock.inventory"

    def action_done(self,cr,uid,ids,context=None):
        for inv in self.browse(cr,uid,ids):
            move_ids = []
            move_line=[]
            for line in inv.inventory_line_id:
                pid=line.product_id.id
                price=line.product_id.standard_price or 0.0
                amount=self.pool.get('stock.location')._product_get(cr, uid, line.location_id.id, [pid], {'uom': line.product_uom.id, 'prodlot': line.prodlot_id.id, 'tracking': line.tracking_id.id})[pid]
                change=line.product_qty-amount
                if change:
                    location_id = line.product_id.product_tmpl_id.property_stock_inventory.id
                    value = {
                        'name': 'INV:'+str(line.inventory_id.id)+':'+line.inventory_id.name,
                        'product_id': line.product_id.id,
                        'product_uom': line.product_uom.id,
                        'prodlot_id': line.prodlot_id.id,
                        'date': inv.date,
                        'date_planned': inv.date,
                        'state': 'assigned'
                    }
                    if change>0:
                        value.update( {
                            'product_qty': change,
                            'location_id': location_id,
                            'location_dest_id': line.location_id.id,
                        })
                    else:
                        value.update( {
                            'product_qty': -change,
                            'location_id': line.location_id.id,
                            'location_dest_id': location_id,
                        })
                    move_ids.append(self.pool.get('stock.move').create(cr, uid, value))
            if len(move_ids):
                self.pool.get('stock.move').action_done(cr, uid, move_ids,
                        context=context)
            self.write(cr, uid, [inv.id], {'state':'done', 'date_done': time.strftime('%Y-%m-%d %H:%M:%S'), 'move_ids': [(6,0,move_ids)]})
        return True
stock_inventory()

class product_product(osv.osv):
    _name="product.product"
    _inherit = "product.product"

    def get_product_available(self,cr,uid,ids,context=None):
        if not context:
            context = {}
        states=context.get('states',[])
        what=context.get('what',())
        if not ids:
            ids = self.search(cr, uid, [])
        res = {}.fromkeys(ids, 0.0)
        if not ids:
            return res

        if context.get('shop', False):
            cr.execute('select warehouse_id from sale_shop where id=%s', (int(context['shop']),))
            res2 = cr.fetchone()
            if res2:
                context['warehouse'] = res2[0]

        if context.get('warehouse', False):
            cr.execute('select lot_stock_id from stock_warehouse where id=%s', (int(context['warehouse']),))
            res2 = cr.fetchone()
            if res2:
                context['location'] = res2[0]

        if context.get('location', False):
            if type(context['location']) == type(1):
                location_ids = [context['location']]
            else:
                location_ids = context['location']
        else:
            cr.execute("select lot_stock_id from stock_warehouse")
            location_ids = [id for (id,) in cr.fetchall()]

        # build the list of ids of children of the location given by id
        if context.get('compute_child',True):
            child_location_ids = self.pool.get('stock.location').search(cr, uid, [('location_id', 'child_of', location_ids)])
            location_ids= len(child_location_ids) and child_location_ids or location_ids
        else:
            location_ids= location_ids

        states_str = ','.join(map(lambda s: "'%s'" % s, states))
        uoms_o = {}
        product2uom = {}
        for product in self.browse(cr, uid, ids, context=context):
            product2uom[product.id] = product.uom_id.id
            uoms_o[product.uom_id.id] = product.uom_id

        prod_ids_str = ','.join(map(str, ids))
        location_ids_str = ','.join(map(str, location_ids))
        results = []
        results2 = []

        if context.get('prodlot'):
            prodlot_id_str='and prodlot_id=%s '%int(context.get('prodlot'))
        else:
            prodlot_id_str=''
        if context.get('tracking'):
            tracking_id_str='and tracking_id=%s '%int(context.get('tracking'))
        else:
            tracking_id_str=''

        from_date=context.get('from_date',False)
        to_date=context.get('to_date',False)
        date_str=False
        if from_date and to_date:
            date_str="date_planned>='%s' and date_planned<='%s'"%(from_date,to_date)
        elif from_date:
            date_str="date_planned>='%s'"%(from_date)
        elif to_date:
            date_str="date_planned<='%s'"%(to_date)

        if 'in' in what:
            # all moves from a location out of the set to a location in the set
            cr.execute(
                'select sum(product_qty), product_id, product_uom '\
                'from stock_move '\
                'where location_id not in ('+location_ids_str+') '\
                'and location_dest_id in ('+location_ids_str+') '\
                'and product_id in ('+prod_ids_str+') '+
                prodlot_id_str+
                tracking_id_str+
                'and state in ('+states_str+') '+ (date_str and 'and '+date_str+' ' or '') +''\
                'group by product_id,product_uom'
            )
            results = cr.fetchall()
        if 'out' in what:
            # all moves from a location in the set to a location out of the set
            cr.execute(
                'select sum(product_qty), product_id, product_uom '\
                'from stock_move '\
                'where location_id in ('+location_ids_str+') '\
                'and location_dest_id not in ('+location_ids_str+') '\
                'and product_id in ('+prod_ids_str+') '+
                prodlot_id_str+
                tracking_id_str+
                'and state in ('+states_str+') '+ (date_str and 'and '+date_str+' ' or '') + ''\
                'group by product_id,product_uom'
            )
            results2 = cr.fetchall()
        uom_obj = self.pool.get('product.uom')
        uoms = map(lambda x: x[2], results) + map(lambda x: x[2], results2)
        if context.get('uom', False):
            uoms += [context['uom']]

        uoms = filter(lambda x: x not in uoms_o.keys(), uoms)
        if uoms:
            uoms = uom_obj.browse(cr, uid, list(set(uoms)), context=context)
        for o in uoms:
            uoms_o[o.id] = o
        for amount, prod_id, prod_uom in results:
            amount = uom_obj._compute_qty_obj(cr, uid, uoms_o[prod_uom], amount,
                    uoms_o[context.get('uom', False) or product2uom[prod_id]])
            res[prod_id] += amount
        for amount, prod_id, prod_uom in results2:
            amount = uom_obj._compute_qty_obj(cr, uid, uoms_o[prod_uom], amount,
                    uoms_o[context.get('uom', False) or product2uom[prod_id]])
            res[prod_id] -= amount
        return res
product_product()
